/*
 * For setting bits on general purpose I/O (gpio) ports.
 *
 *
 * (C) Copyright 2004-2007, Axis Communications AB, LUND, SWEDEN
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include <ctype.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include <syslog.h>
#include "gpio.h"

char gpio_port[] = GPIO_PORT;
unsigned char gpio_bit_count[] = GPIO_BIT_COUNT;

const unsigned int MAX_PORTS = sizeof(gpio_port);

int port_index = -1;
int bit = -1;
int state = -1;

void usage(void) 
{
	unsigned int i;

	fprintf(stderr, "Usage: setbits [OPTIONS]\n");
	fprintf(stderr, "  -p <port>    Which port to use, valid values are %c", gpio_port[0]);
	if (MAX_PORTS == 1) {
		fprintf(stderr, "\n");
	} else {
		for (i = 1; i < MAX_PORTS - 1; i++) {
			fprintf(stderr, ",%c", gpio_port[i]);
		}
		fprintf(stderr, " or %c\n", gpio_port[MAX_PORTS - 1]);
	}
	fprintf(stderr, "  -b <bit>     Bit number to change\n");
	fprintf(stderr, "  -s <state>   Change selected bit to state (0/1)\n");
	exit(1);
}

int get_port_index(char port)
{
	unsigned int i;
	int idx = -1;

	for (i = 0; i < MAX_PORTS; i++) {
		if (gpio_port[i] == port) {
			idx = i;
			break;
		}
        }

	return idx;
}

void
parse_cmdline(int argc, char **argv)
{
	int c;
	unsigned char port;

	while ((c = getopt(argc, argv, "p:b:s:h")) != -1) {
		switch(c) {
		case 'p':
			if (strlen(optarg) == 1) {
				port = tolower(*optarg);

				if ((port_index = get_port_index(port)) < 0) {
					fprintf(stderr, "Invalid port\n");
					usage();
				}	
			} else {
				fprintf(stderr, "Invalid port\n");
				usage();
			}
			break;
		case 'b':
			bit = atoi(optarg);
			break;
		case 's':
			state = atoi(optarg);
			break;
		case 'h':
			usage();
			break;
		
		default:
			printf("Unknown Option... %c\n", c);
			usage();
			break;
		}
	}
}

int main(int argc, char **argv)
{
	int fd;
	char fname[16]; /* Room for /dev/gpiox */
	int bitmask = 1;
	
	parse_cmdline(argc, argv);

	if((bit < 0) || (bit >= gpio_bit_count[port_index])) {
		fprintf(stderr, "Invalid bitnumber: %d\n", bit);
		usage();
	}
	
	if((state < 0) || (state > 1)) {
		fprintf(stderr, "Invalid state: %d\n", state);
		usage();
	}
	
	sprintf(fname, "/dev/gpio%c", gpio_port[port_index]);
	if ((fd = open(fname, O_RDWR)) < 0) {
		int err_saved = errno;
		perror("open");
		fprintf(stderr, "Unable to open device %s. %s", fname, strerror(err_saved));
		exit(1);
	} else {
		bitmask = bitmask << bit;
		if(state) {
			ioctl(fd, _IO(GPIO_IOCTYPE, IO_SETBITS), bitmask); 
		} else {
			ioctl(fd, _IO(GPIO_IOCTYPE, IO_CLRBITS), bitmask); 
		}
	}

	close(fd);
	return 0;
}
